#version 420 core

in vec3 Position_worldspace;
in vec3 Normal_cameraspace;
in vec3 EyeDirection_cameraspace;

uniform vec4 light_position[4];
in vec3 LightDirection_cameraspaces[4];

uniform float ambientIntensity;
uniform float diffuseIntensity;
uniform float specularIntensity;

in vec3 worldNormal;
in vec2 texCoord;

uniform sampler2D material_texture;

out vec4 outputColor;

void main()
{
	vec3 color_final = vec3(0,0,0);
	
	vec3 lightColor = vec3(1, 1, 1);

	for (int i = 0; i < 4; i++) {

		vec3 lightPosition_worldspace = light_position[i].xyz;

		vec3 materialDiffuseColor = texture2D(material_texture, texCoord).rgb;
		vec3 materialAmbientColor = vec3(0.1, 0.1, 0.1) * materialDiffuseColor;
		vec3 materialSpecularColor = vec3(0.3, 0.3, 0.3);

		float distance = length(lightPosition_worldspace - Position_worldspace);

		vec3 n = normalize(Normal_cameraspace);
		vec3 l = normalize(LightDirection_cameraspaces[i]);

		float cosTheta = clamp(dot(n, l), 0, 1);

		vec3 eye = normalize(EyeDirection_cameraspace);
		vec3 reflection = reflect(-l, n);

		float cosAlpha = clamp(dot(eye, reflection), 0, 1);



		color_final += materialAmbientColor *ambientIntensity +
					materialDiffuseColor *diffuseIntensity * lightColor * cosTheta / (distance * distance) +
					materialSpecularColor *specularIntensity * lightColor * pow(cosAlpha, 5) / (distance * distance);
	
	}
	
	outputColor = vec4(color_final, 1.0);
}