#version 330

in VS_OUT {
    vec3 FragPos;
    vec3 Normal;
    vec2 TexCoords;
	vec4 Position_worldspace;
} fs_in;

//textures
uniform sampler2D material_texture;

uniform float ambientIntensity;		
uniform float diffuseIntensity;		 
uniform float specularIntensity;	

uniform vec4 light_position[4]; //xyz, w: lightPower
uniform vec3 camera_position;

// parameters of the material and possible values
uniform vec3	matSpecularReflectanceColor;	// = vec3(1, 1, 1);
uniform float	matShininess;				// = 64;

out vec4 FragColor;

void main()
{
	   vec3 diffuseColor = texture(material_texture, fs_in.TexCoords).rgb;
	   vec3 ambientColor = diffuseColor;
	   vec3 specularColor = matSpecularReflectanceColor;

	   vec3 ambient = ambientColor * ambientIntensity;

	   for(int i = 0; i < 4; i++)
	   {
			 //Diffuse
			vec3 lightDir = normalize(light_position[i].xyz - fs_in.Position_worldspace.xyz);
			vec3 normal = normalize(fs_in.Normal);
			float diff = max(dot(lightDir, normal), 0.0);
			vec3 diffuse = diffuseColor * diff * diffuseIntensity;

			//Specular
			vec3 viewDir = normalize(camera_position - fs_in.Position_worldspace.xyz);
			vec3 reflectDir = reflect(-lightDir, normal);
			float spec = pow(max(dot(viewDir, reflectDir), 0.0), matShininess);
			vec3 specular = vec3(0.3) * spec * specularIntensity; // assuming bright white light color
			
			//attenuation
			float distance = length(light_position[i].xyz - fs_in.Position_worldspace.xyz);
			float attenuation = 1.0 / (distance * distance);

			float lightPower = light_position[i].w;

			FragColor.rgb += ambient + (diffuse + specular) * attenuation * lightPower;
	   }

	   FragColor.a = 1.0f;

	  
}

/*
void main(void)
{
   // normalize vectors after interpolation
   vec3 V = normalize(o_toCamera);
   vec3 N = normalize(o_normal);

   // diffuse color of the object from texture
   vec3 diffuseColor = texture(material_texture, o_texcoords).rgb;
   vec3 ambientColor = diffuseColor;
   vec3 specularColor = matSpecularReflectanceColor;

   // get Blinn-Phong reflectance components
   //ambient
   vec3 ambient = ambientColor * ambientIntensity;

   for(int i = 0; i < 4; i++)
   {
		vec3 L = normalize(o_toLight[i]);
		
		//diffuse 
		float diffuseCoefficient = max(0.0, dot(N, L));
		vec3 diffuse = diffuseColor * diffuseCoefficient *diffuseIntensity;

		//specular
		float specularCoefficient = 0.0;
		if(diffuseCoefficient > 0.0)
			specularCoefficient = pow(max(0.0, dot(V, reflect(-L, N))), matShininess);
		vec3 specular = specularColor * specularCoefficient *  specularIntensity;
    
		//attenuation
		float distance = length(light_position[i] - o_worldPos.xyz);
		float lightAttenuation = 200.0;
		float attenuation = 1.0 / (1.0 + lightAttenuation * pow(distance,2));
		float lightPower = light_position[i].w;

		//linear color (color before gamma correction)
		vec3 linearColor = ambient + attenuation*(diffuse + specular) * lightPower;

		//final color (after gamma correction)
		vec3 gamma = vec3(1.0/2.2);
		outputColor.xyz += pow(linearColor, gamma);
   }

   outputColor.a = 1;
}
*/