#version 420 core

layout(location=0) in vec3 position; //xyz position
layout(location=1) in vec2 texcoord; //xy texture coords
layout(location=2) in vec3 normal; //xyz normal
layout(location=3) in vec3 tangent; //xyz tangent
layout(location=4) in vec3 bitangent; //xyz bitangent

uniform mat4 M_mat;
uniform mat4 P_mat;
uniform mat4 V_mat;
uniform vec4 light_position[4]; //xyz, w: lightPower
uniform vec3 camera_position;

out VS_OUT {
    vec3 FragPos;
    vec2 TexCoords;
    vec4 TangentLightPos1; //cannot pass array -> error
	vec4 TangentLightPos2;
	vec4 TangentLightPos3;
	vec4 TangentLightPos4;
    vec3 TangentViewPos;
    vec3 TangentFragPos;
} vs_out;

/////////////////////////
void main()
{
	gl_Position      = P_mat * V_mat * M_mat * vec4(position, 1.0f);
    vs_out.FragPos   = vec3(M_mat * vec4(position, 1.0));   
    vs_out.TexCoords = texcoord;    
    
    vec3 T   = normalize(mat3(M_mat) * tangent);
    vec3 B   = normalize(mat3(M_mat) * bitangent);
    vec3 N   = normalize(mat3(M_mat) * normal);
    mat3 TBN = transpose(mat3(T, B, N));

	vs_out.TangentLightPos1 = vec4(TBN * light_position[0].xyz, light_position[0].w);
	vs_out.TangentLightPos2 = vec4(TBN * light_position[1].xyz, light_position[1].w);
	vs_out.TangentLightPos3 = vec4(TBN * light_position[2].xyz, light_position[2].w);
    vs_out.TangentLightPos4 = vec4(TBN * light_position[3].xyz, light_position[3].w);

    vs_out.TangentViewPos  = TBN * camera_position;
    vs_out.TangentFragPos  = TBN * vs_out.FragPos;
}

